Package: adbcdrivermanager
Title: 'Arrow' Database Connectivity ('ADBC') Driver Manager
Version: 0.24.0-2
Authors@R: c(
      person("Dewey", "Dunnington", , "dewey@dunnington.ca", role = c("aut", "cre"),
             comment = c(ORCID = "0000-0002-9415-4582")),
      person("Apache Arrow", email = "dev@arrow.apache.org", role = c("aut", "cph")),
      person("Apache Software Foundation", email = "dev@arrow.apache.org", role = c("cph"))
    )
Description: Provides a developer-facing interface to 'Arrow' Database
  Connectivity ('ADBC') for the purposes of driver development, driver
  testing, and building high-level database interfaces for users. 'ADBC'
  <https://arrow.apache.org/adbc/> is an API standard for database access
  libraries that uses 'Arrow' for result sets and query parameters.
License: Apache License (>= 2)
Encoding: UTF-8
Suggests: testthat (>= 3.0.0), withr
Config/testthat/edition: 3
Config/build/bootstrap: TRUE
URL: https://arrow.apache.org/adbc/current/r/adbcdrivermanager/,
        https://github.com/apache/arrow-adbc
BugReports: https://github.com/apache/arrow-adbc/issues
Imports: nanoarrow (>= 0.3.0)
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-08-23 02:42:08 UTC; dewey
Author: Dewey Dunnington [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9415-4582>),
  Apache Arrow [aut, cph],
  Apache Software Foundation [cph]
Maintainer: Dewey Dunnington <dewey@dunnington.ca>
Repository: CRAN
Date/Publication: 2026-08-23 07:40:02 UTC
Built: R 4.6.1; aarch64-apple-darwin23; 2026-08-23 08:59:52 UTC; unix
Archs: adbcdrivermanager.so.dSYM
