# Generated by roxygen2: do not edit by hand

S3method("$",adbc_error)
S3method("$",adbc_xptr)
S3method("$<-",adbc_xptr)
S3method("[[",adbc_error)
S3method("[[",adbc_xptr)
S3method("[[<-",adbc_xptr)
S3method(adbc_connection_init,adbc_database_log)
S3method(adbc_connection_init,adbc_database_monkey)
S3method(adbc_connection_init,default)
S3method(adbc_connection_quote_identifier,default)
S3method(adbc_connection_quote_string,default)
S3method(adbc_database_init,"NULL")
S3method(adbc_database_init,adbc_driver)
S3method(adbc_database_init,adbc_driver_log)
S3method(adbc_database_init,adbc_driver_monkey)
S3method(adbc_database_init,character)
S3method(adbc_statement_init,adbc_connection_log)
S3method(adbc_statement_init,adbc_connection_monkey)
S3method(adbc_statement_init,default)
S3method(execute_adbc,default)
S3method(format,adbc_xptr)
S3method(length,adbc_error)
S3method(length,adbc_xptr)
S3method(names,adbc_error)
S3method(names,adbc_xptr)
S3method(print,adbc_driver)
S3method(print,adbc_error)
S3method(print,adbc_xptr)
S3method(read_adbc,default)
S3method(str,adbc_driver)
S3method(str,adbc_error)
S3method(str,adbc_xptr)
S3method(write_adbc,default)
export(adbc_connection_cancel)
export(adbc_connection_commit)
export(adbc_connection_get_info)
export(adbc_connection_get_objects)
export(adbc_connection_get_option)
export(adbc_connection_get_option_bytes)
export(adbc_connection_get_option_double)
export(adbc_connection_get_option_int)
export(adbc_connection_get_statistic_names)
export(adbc_connection_get_statistics)
export(adbc_connection_get_table_schema)
export(adbc_connection_get_table_types)
export(adbc_connection_init)
export(adbc_connection_init_default)
export(adbc_connection_join)
export(adbc_connection_quote_identifier)
export(adbc_connection_quote_string)
export(adbc_connection_read_partition)
export(adbc_connection_release)
export(adbc_connection_rollback)
export(adbc_connection_set_options)
export(adbc_database_get_option)
export(adbc_database_get_option_bytes)
export(adbc_database_get_option_double)
export(adbc_database_get_option_int)
export(adbc_database_init)
export(adbc_database_init_default)
export(adbc_database_release)
export(adbc_database_set_options)
export(adbc_driver)
export(adbc_driver_load)
export(adbc_driver_log)
export(adbc_driver_monkey)
export(adbc_driver_void)
export(adbc_error_from_array_stream)
export(adbc_statement_bind)
export(adbc_statement_bind_stream)
export(adbc_statement_cancel)
export(adbc_statement_execute_query)
export(adbc_statement_execute_schema)
export(adbc_statement_get_option)
export(adbc_statement_get_option_bytes)
export(adbc_statement_get_option_double)
export(adbc_statement_get_option_int)
export(adbc_statement_get_parameter_schema)
export(adbc_statement_init)
export(adbc_statement_init_default)
export(adbc_statement_join)
export(adbc_statement_prepare)
export(adbc_statement_release)
export(adbc_statement_set_options)
export(adbc_statement_set_sql_query)
export(adbc_statement_set_substrait_plan)
export(adbc_xptr_is_valid)
export(adbc_xptr_move)
export(execute_adbc)
export(local_adbc)
export(read_adbc)
export(with_adbc)
export(write_adbc)
importFrom(utils,str)
useDynLib(adbcdrivermanager, .registration = TRUE)
